/* ftUDP_server.c
 * Serveur UDP pour le client ftUDP_client permettant d'afficher
 * le contenu d'un fichier envoy par le client.
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 16 octobre 2002
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

/* Fonction affichant un message d'erreur en fonction de la valeur
 * de errno et quitte le programme.
 */
void erreur(char *s)
{
	perror(s);
	exit(-1);
}

#define BUFSIZE 100	// taille du buffer
int main(int argc, char **argv)
{
	struct sockaddr_in addr;	// adresse du serveur puis adresse du client
	int sock;			// socket
	int port;			// port
	char buf[BUFSIZE+1];		// buffer
	int l;				// taille d'une structure
	int nbuf;			// nombre de char recus
	
	/* Test des arguments de la ligne de commande */
	if (argc != 2) {
		printf("Usage: ftUDP_server port\n");
		exit(0);
	}
	
	/* Cration du socket */
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock < 0) erreur("socket");
	
	/* Test de la validit du port */
	port = atoi(argv[1]);
	if (port < 1024 || port > 65000) {
		printf("Numro de port invalide: 1024 < port < 65000\n");
		exit(-1);
	}
	
	/* Cration de la structure sockaddr_in (adresse du serveur) */
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);	// port
	addr.sin_addr.s_addr = htonl(INADDR_ANY);	
	memset(&(addr.sin_zero), '\0', 8);	// on met  zro le reste de la struct
	
	l = sizeof(struct sockaddr);	// taille de la structure sockaddr
	
	/* Attacher une adresse au socket */
	if (bind(sock, (struct sockaddr*) &addr, l) < 0) erreur("bind");
	
	/* Boucle du serveur */
	while (1) {
		// Rcupre les packets envoys par le client de la structure addr
		if ((nbuf = recvfrom(sock, buf, BUFSIZE, 0, (struct sockaddr*) &addr, &l)) < 0) 
			erreur("recvfrom");
		buf[nbuf] = '\0';	// on termine la chaine de caractres recues par \0
					// pour permettre un affichage correct par printf
		printf("%s", buf);
	}
	
	return 1;
}
